/*	$OpenBSD: flex.skl,v 1.6 2001/01/05 18:26:23 millert Exp $	*/

/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /cvs/Darwin/sudo/sudo/lex.yy.c,v 1.1.1.3 2002/02/04 19:35:52 bbraun Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>
#include <errno.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#ifdef __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart YY_PROTO(( FILE *input_file ));

void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *yy_str ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len ));

static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)

typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 35
#define YY_END_OF_BUFFER 36
static yyconst short int yy_accept[299] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   36,   25,   31,   30,   29,   34,   25,   20,
       34,   25,   26,   25,   25,   25,   25,   28,   27,   21,
       21,   21,   34,   21,   21,   21,   21,   21,   22,   34,
       22,   23,   22,   22,   22,   22,   22,   21,   21,   21,
       34,    1,   11,   10,   11,   10,   10,   34,   34,    2,
        8,    8,    8,    3,    8,    4,   34,   25,    0,   31,
       29,    0,   33,   17,    0,   16,    0,   24,   24,    0,
       25,   25,   25,   25,   25,   21,   21,   21,   25,   32,
       25,   25,   25,   25,   25,   22,    0,   22,   17,    0,

       16,    0,   22,    0,   22,   22,   22,   22,   22,   21,
       21,   21,   22,    1,   11,   11,    9,    9,    0,    2,
        8,    0,    8,    0,    0,    5,    6,    8,    8,    0,
       25,   25,   25,   21,   21,   25,   25,   25,   25,   25,
       22,   22,   22,   21,   21,    7,    7,    0,    7,    8,
       25,   25,   25,   25,   25,   21,   21,   25,   25,   25,
       22,   22,   22,   22,   22,   21,   21,    7,   25,   25,
       25,   21,   21,   25,   25,   22,   22,   22,   21,   21,
       25,   25,   25,   25,   25,   21,   21,   25,   25,   22,
       22,   22,   22,   22,   21,   21,   18,   18,   18,   21,

        0,   15,   25,   25,   18,   18,   18,   21,   25,   18,
       18,   18,   18,   21,   25,   12,   22,   18,   18,   18,
       18,   21,   25,   19,   19,   19,    0,   14,   25,   12,
       22,   19,   19,   19,   25,   25,   19,   19,   19,   19,
       19,   13,   22,   22,   19,   19,   19,   19,   19,   25,
       25,   25,   19,   19,   22,   22,   22,   19,   19,   25,
       25,   25,   25,   25,   22,   22,   22,   22,   22,   25,
       25,   25,   22,   22,   22,   25,   25,   25,   25,   25,
       22,   22,   22,   22,   22,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   18,   18,   18,    0

    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    1,    7,    1,    1,    8,
        9,    1,   10,   11,   12,   13,   14,   15,   16,   17,
       18,   19,   20,   21,   21,   21,   21,   22,    1,    1,
       23,    1,    1,   24,   25,   26,   27,   28,   26,   26,
       26,   29,   26,   26,   26,   26,   26,   30,   31,   32,
       26,   33,   34,   26,   35,   26,   36,   26,   26,   26,
        1,   37,    1,    1,   38,    1,   39,   40,   40,   41,

       42,   43,   40,   40,   44,   40,   40,   45,   46,   47,
       48,   40,   40,   49,   50,   51,   52,   40,   40,   40,
       40,   40,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[53] =
    {   0,
        1,    2,    2,    3,    1,    4,    1,    3,    3,    1,
        2,    5,    1,    1,    5,    5,    5,    5,    5,    5,
        5,    6,    4,    3,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    7,    8,    8,    8,
        8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
        8,    8
    } ;

static yyconst short int yy_base[319] =
    {   0,
        0,   26,   61,    0,  112,  113,  120,  156,  192,  243,
      294,  330,  627,  589,  623, 1808,  619,  619,  584, 1808,
     1808,  583, 1808,  124,  355,  119,  134, 1808, 1808,  380,
      586,  579,  417,  557,  558,  551,  546,  545,  556,    0,
      553, 1808,  551,  160,  429,  155,  170,  454,  555,  545,
      491,  565,    0, 1808,  561,    0, 1808,  307,   54,    0,
      526,  343,   93, 1808,  104, 1808,  142,  521,  514,  555,
      552,  544, 1808,  507,  537,  505,  560,  315,  503,  583,
      593,  143,    0,    0,  503,    0,  489,  485,  144, 1808,
      118,  140,  154,  145,  163,  480,  629,    0,  475,  652,

      474,  675,  353,  698,  708,  186,    0,    0,  489,    0,
      477,  467,  211,  496,    0,  492,  199, 1808,  204,    0,
      431,  248,  380,  214,  215, 1808, 1808,  427,  219,  251,
      733,  742,  751,  438,  428,   88,   21,  166,  171,  175,
      760,  769,  778,  436,  421,  416, 1808,  221,  454,  613,
      785,  242,  810,  819,  828,  409,  386,  178,  223,  226,
      837,  264,  846,  855,  864,  360,  345,  265,  873,  882,
      891,  335,  328,  241,  224,  900,  909,  918,  321,  323,
      925,  290,  950,  959,  968,  311,  306,  227,  174,  977,
      301,  986,  995, 1004,  285,  341, 1012,  414,  488,  288,

      356, 1808,  278,  270, 1037,  511,  534,  283, 1044,  323,
        0,    0,  247,  357,  228,  365, 1068,  351,    0,    0,
      207,  362, 1077, 1086, 1095, 1104,  402, 1808,  275, 1808,
     1113, 1122, 1131, 1140, 1147,  313, 1172, 1181, 1190, 1199,
      559,  190, 1208,  414, 1217, 1226, 1235, 1244,  564, 1253,
     1262, 1271, 1280,  421, 1289, 1298, 1307, 1316,  518, 1323,
      439, 1348, 1357, 1366, 1373,  441, 1398, 1407, 1416, 1425,
     1434, 1443, 1452, 1461, 1470, 1479,  560, 1488, 1497, 1506,
     1515,  565, 1524, 1533, 1542, 1551, 1560, 1569, 1578, 1587,
     1596,  172, 1603,  627,  129, 1626,  650, 1808, 1663, 1671,

     1679, 1687, 1695, 1703, 1711, 1719, 1727,  158, 1735, 1743,
     1751, 1759,  109, 1767, 1775, 1783, 1791, 1799
    } ;

static yyconst short int yy_def[319] =
    {   0,
      298,    1,  298,    3,    1,    1,  299,  299,  300,  300,
      301,  301,  298,  302,  298,  298,  298,  303,  304,  298,
      298,  305,  298,  306,  302,   25,   25,  298,  298,   25,
       30,   30,  302,   30,   30,   30,   30,   30,  307,  308,
      309,  298,  310,  311,  307,   45,   45,   45,   48,   48,
      307,  298,  312,  298,  312,  312,  298,  298,  298,  313,
      314,  315,  314,  298,  314,  298,  316,  302,  302,  298,
      298,  303,  298,  304,  304,  305,  305,  306,  317,  302,
      302,  302,   25,   25,   25,   30,   30,   30,  302,  298,
      302,  302,  302,  302,  302,  307,  307,  308,  309,  309,

      310,  310,  311,  307,  307,  307,   45,   45,   45,   48,
       48,   48,  307,  298,  312,  312,  298,  298,  298,  313,
      314,  314,  315,  318,  315,  298,  298,  314,  314,  298,
       25,   25,   25,   30,   30,  302,  302,  302,  302,  302,
       45,   45,   45,   48,   48,  314,  298,  318,  315,  315,
      302,  302,   25,   25,   25,   30,   30,  302,  302,  302,
       45,  307,   45,   45,   45,   48,   48,  318,   25,   25,
       25,   30,   30,  302,  302,   45,   45,   45,   48,   48,
      302,  302,   25,   25,   25,   30,   30,  302,  302,   45,
      307,   45,   45,   45,   48,   48,  302,  197,  197,   30,

      298,  298,  302,  302,   45,  205,  205,   48,  302,  302,
      197,  197,  197,   30,  302,  302,  205,  307,  205,  205,
      205,   48,   25,   25,   25,   25,  298,  298,  302,  298,
       45,   45,   45,   45,  302,  302,   25,   25,   25,   25,
      302,  302,   45,  307,   45,   45,   45,   45,  307,   25,
       25,   25,   25,  302,   45,   45,   45,   45,  307,  302,
      302,   25,   25,   25,  307,  307,   45,   45,   45,   25,
       25,   25,   45,   45,   45,   25,  302,   25,   25,   25,
       45,  307,   45,   45,   45,   25,   25,   25,   45,   45,
       45,  302,  302,  302,  307,  307,  307,    0,  298,  298,

      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298
    } ;

static yyconst short int yy_nxt[1861] =
    {   0,
       14,   15,   16,   17,   14,   18,   19,   20,   21,   22,
       23,   14,   14,   24,   25,   26,   27,   25,   25,   25,
       25,   28,   29,   21,   30,   30,   30,   30,   30,   31,
       30,   32,   30,   30,   30,   30,   33,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   34,   35,   36,  119,   90,   69,   37,  159,
       38,   39,   15,   16,   17,   39,   40,   41,   21,   42,
       43,   23,   39,   39,   44,   45,   46,   47,   45,   45,
       45,   45,   28,   29,   21,   48,   48,   48,   48,   48,
       49,   48,   50,   48,   48,   48,   48,   51,   39,   39,

       39,   39,   39,   39,   39,   39,   39,   39,   39,   39,
       39,   39,   39,   52,   52,  126,  120,   21,   21,   21,
       21,   15,   54,   55,   69,   56,  127,   79,  158,  122,
       57,   79,   79,   83,   83,   83,   83,   83,   83,   83,
      122,   57,   56,  129,   90,  119,   90,   79,   84,   84,
       84,   84,   84,   85,   69,   81,   58,   15,   54,   55,
       80,   56,   98,   79,  136,   97,   57,   79,   79,  107,
      107,  107,  107,  107,  107,  107,   69,   57,   56,   69,
       69,   69,  137,   79,  108,  108,  108,  108,  108,  109,
       69,  139,   58,   15,   16,   17,  104,   18,  105,   69,

      119,   90,   69,  138,  140,  119,   90,   69,   69,  160,
       69,   69,  119,   90,   69,  174,  158,  124,  147,  149,
      119,   90,   97,  158,  204,  168,   69,   96,   59,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   15,   16,   17,   97,   18,  121,
      148,  150,   79,  121,  151,  122,   79,  148,  121,   69,
       69,   79,   69,   69,   69,  188,  229,   68,  189,  147,
      121,  203,   79,   79,  175,  158,  161,   69,   69,   59,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   60,   15,   16,   17,   62,   18,

       97,  148,  181,   63,   64,   65,   69,  201,  117,   90,
      222,   69,  118,  190,   69,  214,   66,  118,   79,  216,
      208,  215,   79,   79,  242,  235,   69,  202,  118,  118,
       67,   15,   16,   17,   62,   18,  209,   97,   79,   63,
       64,   65,  201,  118,  124,  124,  200,  121,  124,   69,
      196,   80,   66,  124,  195,  187,   79,  201,  227,   69,
       79,   79,  202,  227,  217,  124,   67,   81,  186,   82,
       82,   82,   82,   82,   82,   82,   79,  202,  228,  125,
      180,  124,  124,  228,  146,  124,  230,   97,  230,  104,
      124,   69,   68,  179,   86,   86,   86,   86,   86,   86,

       86,   69,  124,  227,   86,   86,   86,   86,   86,   86,
       86,   86,   86,   86,   86,   86,  125,   86,   89,   90,
       68,  173,   68,  228,   68,   68,  243,   68,  211,  211,
      211,  211,  211,  211,  211,  254,  254,  254,   68,   68,
       68,  105,  172,  106,  106,  106,  106,  106,  106,  106,
       97,  260,  122,  265,  167,  124,  124,   69,  146,  124,
      166,  157,  156,  122,  124,   97,   96,  122,  110,  110,
      110,  110,  110,  110,  110,   69,  124,   97,  110,  110,
      110,  110,  110,  110,  110,  110,  110,  110,  110,  110,
      125,  110,  113,   90,   96,  116,   96,  114,   96,   96,

      145,   96,  212,  212,  212,  212,  212,  213,  144,   96,
      102,  100,   96,   96,   96,   68,   97,   68,  135,   68,
      134,   68,   68,   68,   68,  219,  219,  219,  219,  219,
      219,  219,  259,  259,  259,   68,   68,   68,   74,  130,
       74,   77,   74,   75,   74,   74,   73,   74,  220,  220,
      220,  220,  220,  221,   97,   71,   70,   69,   74,   74,
       74,   76,  122,   76,  116,   76,  114,   76,   76,  112,
       76,  235,  276,  254,  254,  254,  243,  281,  259,  259,
      259,   76,   76,   76,   78,  111,   68,  102,   78,  100,
       68,   68,   97,   78,   95,   69,   69,   94,   93,   92,

       97,   97,   91,   88,   78,   78,   68,  131,  132,  133,
      131,  131,  131,  131,  124,  124,   87,  149,  124,   77,
       75,   73,   71,  124,   70,   69,  298,  298,  298,   69,
       96,  298,   96,  298,   96,  124,   96,   96,  298,   96,
      298,  292,  292,  292,  292,  292,  292,  298,  298,  125,
       96,   96,   96,   99,  298,   99,  298,   99,  298,   99,
       99,  298,   99,   69,  295,  295,  295,  295,  295,  295,
      298,  298,  298,   99,   99,   99,  101,  298,  101,  298,
      101,  298,  101,  101,  298,  101,   97,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  101,  101,  101,  103,

      298,   96,  298,  103,  298,   96,   96,  298,  103,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  103,
      103,   96,  141,  142,  143,  141,  141,  141,  141,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,   97,  151,  298,  152,  152,  152,
      152,  152,  152,  152,  151,  298,  153,  153,  153,  153,
      153,  153,  153,  151,  298,  154,  154,  154,  154,  154,
      155,  152,  161,  298,  162,  162,  162,  162,  162,  162,
      162,  161,  298,  163,  163,  163,  163,  163,  163,  163,
      161,  298,  164,  164,  164,  164,  164,  165,  162,  169,

      170,  171,  169,  169,  169,  169,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,   69,  151,  298,  152,  152,  152,  152,  152,  152,
      152,  151,  298,  152,  152,  152,  152,  152,  152,  152,
      151,  298,  152,  152,  152,  152,  152,  152,   68,   96,
      298,  176,  177,  178,  176,  176,  176,  176,  161,  298,
      162,  162,  162,  162,  162,  162,  162,  161,  298,  162,
      162,  162,  162,  162,  162,  162,  161,  298,  162,  162,
      162,  162,  162,  162,   96,  181,  298,  182,  182,  182,
      182,  182,  182,  182,  181,  298,  183,  183,  183,  183,

      183,  183,  183,  181,  298,  184,  184,  184,  184,  184,
      185,  182,  190,  298,  191,  191,  191,  191,  191,  191,
      191,  190,  298,  192,  192,  192,  192,  192,  192,  192,
      190,  298,  193,  193,  193,  193,  193,  194,  191,  197,
      198,  199,  197,  197,  197,  197,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,   69,  181,  298,  182,  182,  182,  182,  182,  182,
      182,  181,  298,  182,  182,  182,  182,  182,  182,  182,
      181,  298,  182,  182,  182,  182,  182,  182,   68,   96,
      298,  205,  206,  207,  205,  205,  205,  205,  190,  298,

      191,  191,  191,  191,  191,  191,  191,  190,  298,  191,
      191,  191,  191,  191,  191,  191,  190,  298,  191,  191,
      191,  191,  191,  191,   96,  209,  210,  210,  210,  210,
      210,  210,  210,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,   69,   96,
      217,  218,  218,  218,  218,  218,  218,  218,  223,  224,
      225,  226,  223,  223,  223,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
       69,   96,  231,  232,  233,  234,  231,  231,  231,  235,
      298,  236,  236,  236,  236,  236,  236,  236,  235,  298,

      237,  237,  237,  237,  237,  237,  237,  235,  298,  238,
      238,  238,  238,  238,  239,  240,  235,  298,  241,  241,
      241,  236,  236,  236,  236,  243,  298,  244,  244,  244,
      244,  244,  244,  244,  243,  298,  245,  245,  245,  245,
      245,  245,  245,  243,  298,  246,  246,  246,  246,  246,
      247,  248,  243,  298,  249,  249,  249,  244,  244,  244,
      244,  250,  251,  252,  250,  250,  250,  250,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,   69,  235,  298,  240,  240,  240,  240,
      240,  240,  240,  235,  298,  240,  240,  240,  240,  240,

      240,  240,  235,  298,  240,  240,  240,  240,  240,  240,
      253,  235,  298,  253,  253,  253,  253,  253,  253,  253,
       96,  298,  255,  256,  257,  255,  255,  255,  255,  243,
      298,  248,  248,  248,  248,  248,  248,  248,  243,  298,
      248,  248,  248,  248,  248,  248,  248,  243,  298,  248,
      248,  248,  248,  248,  248,  258,  243,  298,  258,  258,
      258,  258,  258,  258,  258,  260,  298,  261,  261,  261,
      261,  261,  261,  261,  260,  298,  262,  262,  262,  262,
      262,  262,  262,  260,  298,  263,  263,  263,  263,  263,
      264,  261,   68,  298,  253,  253,  253,  253,  253,  253,

      253,  265,  298,  266,  266,  266,  266,  266,  266,  266,
      265,  298,  267,  267,  267,  267,  267,  267,  267,  265,
      298,  268,  268,  268,  268,  268,  269,  266,   96,  298,
      258,  258,  258,  258,  258,  258,  258,  270,  271,  272,
      270,  270,  270,  270,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,   69,
      260,  298,  261,  261,  261,  261,  261,  261,  261,  260,
      298,  261,  261,  261,  261,  261,  261,  261,  260,  298,
      261,  261,  261,  261,  261,  261,   68,  273,  274,  275,
      273,  273,  273,  273,  298,  298,  298,  298,  298,  298,

      298,  298,  298,  298,  298,  298,  298,  298,  298,   97,
      265,  298,  266,  266,  266,  266,  266,  266,  266,  265,
      298,  266,  266,  266,  266,  266,  266,  266,  265,  298,
      266,  266,  266,  266,  266,  266,   96,  276,  298,  277,
      277,  277,  277,  277,  277,  277,  276,  298,  278,  278,
      278,  278,  278,  278,  278,  276,  298,  279,  279,  279,
      279,  279,  280,  277,  281,  298,  282,  282,  282,  282,
      282,  282,  282,  281,  298,  283,  283,  283,  283,  283,
      283,  283,  281,  298,  284,  284,  284,  284,  284,  285,
      282,   68,  298,  286,  287,  288,  286,  286,  286,  286,

      276,  298,  277,  277,  277,  277,  277,  277,  277,  276,
      298,  277,  277,  277,  277,  277,  277,  277,  276,  298,
      277,  277,  277,  277,  277,  277,   68,   96,  298,  289,
      290,  291,  289,  289,  289,  289,  281,  298,  282,  282,
      282,  282,  282,  282,  282,  281,  298,  282,  282,  282,
      282,  282,  282,  282,  281,  298,  282,  282,  282,  282,
      282,  282,   96,   68,  298,  292,  292,  292,  292,  292,
      292,  292,   68,  298,  286,  286,  286,  286,  286,  286,
      286,   68,  298,  293,  293,  293,  293,  293,  294,  292,
       96,  298,  295,  295,  295,  295,  295,  295,  295,   96,

      298,  289,  289,  289,  289,  289,  289,  289,   96,  298,
      296,  296,  296,  296,  296,  297,  295,  292,  292,  292,
      292,  292,  292,  292,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,   69,
      295,  295,  295,  295,  295,  295,  295,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,   97,   53,   53,   53,   53,   53,   53,   53,
       53,   21,   21,   21,   21,   21,   21,   21,   21,   61,
       61,   61,   61,   61,   61,   61,   61,   68,  298,  298,
      298,   68,  298,   68,   68,   72,   72,   72,   72,   72,

       72,   72,   72,   74,  298,  298,  298,   74,  298,   74,
       74,   76,  298,  298,  298,   76,  298,   76,   76,   78,
      298,   78,  298,   78,  298,   78,   78,   96,  298,  298,
      298,   96,  298,   96,   96,   99,  298,  298,  298,   99,
      298,   99,   99,  101,  298,  298,  298,  101,  298,  101,
      101,  103,  298,  103,  298,  103,  298,  103,  103,  115,
      298,  115,  115,  115,  298,  298,  115,  121,  298,  121,
      298,  121,  121,  121,  121,  123,  123,  123,  123,  123,
      123,  123,  123,  128,  128,  128,  128,  128,  128,  128,
      128,   79,  298,   79,  298,   79,  298,   79,   79,  124,

      124,  124,  124,  124,  124,  124,  124,   13,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298
    } ;

static yyconst short int yy_chk[1861] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    2,    2,    2,   59,   59,  137,    2,  137,
        2,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,

        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    5,    6,   63,  313,    5,    6,    5,
        6,    7,    7,    7,  136,    7,   65,   24,  136,   63,
        7,   24,   24,   26,   26,   26,   26,   26,   26,   26,
       65,    7,    7,   67,   67,   89,   89,   24,   27,   27,
       27,   27,   27,   27,   91,   82,    7,    8,    8,    8,
       24,    8,  308,   44,   91,  295,    8,   44,   44,   46,
       46,   46,   46,   46,   46,   46,   92,    8,    8,   82,
       89,   94,   92,   44,   47,   47,   47,   47,   47,   47,
       93,   94,    8,    9,    9,    9,   44,    9,  106,   95,

      117,  117,  138,   93,   95,  119,  119,  139,  292,  139,
      189,  140,  113,  113,  158,  158,  138,  125,  124,  125,
      129,  129,  106,  140,  189,  148,  242,  221,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,   10,   10,   10,  113,   10,  122,
      124,  125,  130,  122,  152,  129,  130,  148,  122,  159,
      175,  130,  160,  188,  215,  174,  215,  213,  175,  168,
      122,  188,  130,  130,  159,  160,  162,  174,  152,   10,
       10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10,   10,   10,   10,   10,   11,   11,   11,   11,   11,

      162,  168,  182,   11,   11,   11,  204,  187,   58,   58,
      208,  229,   58,  191,  203,  200,   11,   58,   78,  204,
      195,  203,   78,   78,  229,  236,  182,  187,   58,   58,
       11,   12,   12,   12,   12,   12,  210,  191,   78,   12,
       12,   12,  196,   58,   62,   62,  186,   62,   62,  236,
      180,   78,   12,   62,  179,  173,  103,  201,  214,  210,
      103,  103,  196,  222,  218,   62,   12,   25,  172,   25,
       25,   25,   25,   25,   25,   25,  103,  201,  214,   62,
      167,  123,  123,  222,  123,  123,  216,  218,  216,  103,
      123,   25,   30,  166,   30,   30,   30,   30,   30,   30,

       30,  216,  123,  227,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   30,  123,   30,   33,   33,
       33,  157,   33,  227,   33,   33,  244,   33,  198,  198,
      198,  198,  198,  198,  198,  254,  254,  254,   33,   33,
       33,   45,  156,   45,   45,   45,   45,   45,   45,   45,
      244,  261,  146,  266,  145,  149,  149,  254,  149,  149,
      144,  135,  134,  128,  149,   45,   48,  121,   48,   48,
       48,   48,   48,   48,   48,  261,  149,  266,   48,   48,
       48,   48,   48,   48,   48,   48,   48,   48,   48,   48,
      149,   48,   51,   51,   51,  116,   51,  114,   51,   51,

      112,   51,  199,  199,  199,  199,  199,  199,  111,  109,
      101,   99,   51,   51,   51,   69,   96,   69,   88,   69,
       87,   69,   69,   85,   69,  206,  206,  206,  206,  206,
      206,  206,  259,  259,  259,   69,   69,   69,   75,   79,
       75,   76,   75,   74,   75,   75,   72,   75,  207,  207,
      207,  207,  207,  207,  259,   71,   70,   68,   75,   75,
       75,   77,   61,   77,   55,   77,   52,   77,   77,   50,
       77,  241,  277,  241,  241,  241,  249,  282,  249,  249,
      249,   77,   77,   77,   80,   49,   80,   43,   80,   41,
       80,   80,   39,   80,   38,  241,  277,   37,   36,   35,

      249,  282,   34,   32,   80,   80,   80,   81,   81,   81,
       81,   81,   81,   81,  150,  150,   31,  150,  150,   22,
       19,   18,   17,  150,   15,   14,   13,    0,    0,   81,
       97,    0,   97,    0,   97,  150,   97,   97,    0,   97,
        0,  294,  294,  294,  294,  294,  294,    0,    0,  150,
       97,   97,   97,  100,    0,  100,    0,  100,    0,  100,
      100,    0,  100,  294,  297,  297,  297,  297,  297,  297,
        0,    0,    0,  100,  100,  100,  102,    0,  102,    0,
      102,    0,  102,  102,    0,  102,  297,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  102,  102,  102,  104,

        0,  104,    0,  104,    0,  104,  104,    0,  104,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  104,
      104,  104,  105,  105,  105,  105,  105,  105,  105,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,  105,  131,    0,  131,  131,  131,
      131,  131,  131,  131,  132,    0,  132,  132,  132,  132,
      132,  132,  132,  133,    0,  133,  133,  133,  133,  133,
      133,  133,  141,    0,  141,  141,  141,  141,  141,  141,
      141,  142,    0,  142,  142,  142,  142,  142,  142,  142,
      143,    0,  143,  143,  143,  143,  143,  143,  143,  151,

      151,  151,  151,  151,  151,  151,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,  151,  153,    0,  153,  153,  153,  153,  153,  153,
      153,  154,    0,  154,  154,  154,  154,  154,  154,  154,
      155,    0,  155,  155,  155,  155,  155,  155,  155,  161,
        0,  161,  161,  161,  161,  161,  161,  161,  163,    0,
      163,  163,  163,  163,  163,  163,  163,  164,    0,  164,
      164,  164,  164,  164,  164,  164,  165,    0,  165,  165,
      165,  165,  165,  165,  165,  169,    0,  169,  169,  169,
      169,  169,  169,  169,  170,    0,  170,  170,  170,  170,

      170,  170,  170,  171,    0,  171,  171,  171,  171,  171,
      171,  171,  176,    0,  176,  176,  176,  176,  176,  176,
      176,  177,    0,  177,  177,  177,  177,  177,  177,  177,
      178,    0,  178,  178,  178,  178,  178,  178,  178,  181,
      181,  181,  181,  181,  181,  181,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,  181,  183,    0,  183,  183,  183,  183,  183,  183,
      183,  184,    0,  184,  184,  184,  184,  184,  184,  184,
      185,    0,  185,  185,  185,  185,  185,  185,  185,  190,
        0,  190,  190,  190,  190,  190,  190,  190,  192,    0,

      192,  192,  192,  192,  192,  192,  192,  193,    0,  193,
      193,  193,  193,  193,  193,  193,  194,    0,  194,  194,
      194,  194,  194,  194,  194,  197,  197,  197,  197,  197,
      197,  197,  197,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,  197,  205,
      205,  205,  205,  205,  205,  205,  205,  205,  209,  209,
      209,  209,  209,  209,  209,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
      209,  217,  217,  217,  217,  217,  217,  217,  217,  223,
        0,  223,  223,  223,  223,  223,  223,  223,  224,    0,

      224,  224,  224,  224,  224,  224,  224,  225,    0,  225,
      225,  225,  225,  225,  225,  225,  226,    0,  226,  226,
      226,  226,  226,  226,  226,  231,    0,  231,  231,  231,
      231,  231,  231,  231,  232,    0,  232,  232,  232,  232,
      232,  232,  232,  233,    0,  233,  233,  233,  233,  233,
      233,  233,  234,    0,  234,  234,  234,  234,  234,  234,
      234,  235,  235,  235,  235,  235,  235,  235,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,  235,  237,    0,  237,  237,  237,  237,
      237,  237,  237,  238,    0,  238,  238,  238,  238,  238,

      238,  238,  239,    0,  239,  239,  239,  239,  239,  239,
      239,  240,    0,  240,  240,  240,  240,  240,  240,  240,
      243,    0,  243,  243,  243,  243,  243,  243,  243,  245,
        0,  245,  245,  245,  245,  245,  245,  245,  246,    0,
      246,  246,  246,  246,  246,  246,  246,  247,    0,  247,
      247,  247,  247,  247,  247,  247,  248,    0,  248,  248,
      248,  248,  248,  248,  248,  250,    0,  250,  250,  250,
      250,  250,  250,  250,  251,    0,  251,  251,  251,  251,
      251,  251,  251,  252,    0,  252,  252,  252,  252,  252,
      252,  252,  253,    0,  253,  253,  253,  253,  253,  253,

      253,  255,    0,  255,  255,  255,  255,  255,  255,  255,
      256,    0,  256,  256,  256,  256,  256,  256,  256,  257,
        0,  257,  257,  257,  257,  257,  257,  257,  258,    0,
      258,  258,  258,  258,  258,  258,  258,  260,  260,  260,
      260,  260,  260,  260,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  260,
      262,    0,  262,  262,  262,  262,  262,  262,  262,  263,
        0,  263,  263,  263,  263,  263,  263,  263,  264,    0,
      264,  264,  264,  264,  264,  264,  264,  265,  265,  265,
      265,  265,  265,  265,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,  265,
      267,    0,  267,  267,  267,  267,  267,  267,  267,  268,
        0,  268,  268,  268,  268,  268,  268,  268,  269,    0,
      269,  269,  269,  269,  269,  269,  269,  270,    0,  270,
      270,  270,  270,  270,  270,  270,  271,    0,  271,  271,
      271,  271,  271,  271,  271,  272,    0,  272,  272,  272,
      272,  272,  272,  272,  273,    0,  273,  273,  273,  273,
      273,  273,  273,  274,    0,  274,  274,  274,  274,  274,
      274,  274,  275,    0,  275,  275,  275,  275,  275,  275,
      275,  276,    0,  276,  276,  276,  276,  276,  276,  276,

      278,    0,  278,  278,  278,  278,  278,  278,  278,  279,
        0,  279,  279,  279,  279,  279,  279,  279,  280,    0,
      280,  280,  280,  280,  280,  280,  280,  281,    0,  281,
      281,  281,  281,  281,  281,  281,  283,    0,  283,  283,
      283,  283,  283,  283,  283,  284,    0,  284,  284,  284,
      284,  284,  284,  284,  285,    0,  285,  285,  285,  285,
      285,  285,  285,  286,    0,  286,  286,  286,  286,  286,
      286,  286,  287,    0,  287,  287,  287,  287,  287,  287,
      287,  288,    0,  288,  288,  288,  288,  288,  288,  288,
      289,    0,  289,  289,  289,  289,  289,  289,  289,  290,

        0,  290,  290,  290,  290,  290,  290,  290,  291,    0,
      291,  291,  291,  291,  291,  291,  291,  293,  293,  293,
      293,  293,  293,  293,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  293,
      296,  296,  296,  296,  296,  296,  296,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,  296,  299,  299,  299,  299,  299,  299,  299,
      299,  300,  300,  300,  300,  300,  300,  300,  300,  301,
      301,  301,  301,  301,  301,  301,  301,  302,    0,    0,
        0,  302,    0,  302,  302,  303,  303,  303,  303,  303,

      303,  303,  303,  304,    0,    0,    0,  304,    0,  304,
      304,  305,    0,    0,    0,  305,    0,  305,  305,  306,
        0,  306,    0,  306,    0,  306,  306,  307,    0,    0,
        0,  307,    0,  307,  307,  309,    0,    0,    0,  309,
        0,  309,  309,  310,    0,    0,    0,  310,    0,  310,
      310,  311,    0,  311,    0,  311,    0,  311,  311,  312,
        0,  312,  312,  312,    0,    0,  312,  314,    0,  314,
        0,  314,  314,  314,  314,  315,  315,  315,  315,  315,
      315,  315,  315,  316,  316,  316,  316,  316,  316,  316,
      316,  317,    0,  317,    0,  317,    0,  317,  317,  318,

      318,  318,  318,  318,  318,  318,  318,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298,
      298,  298,  298,  298,  298,  298,  298,  298,  298,  298
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "parse.lex"
#define INITIAL 0
#line 2 "parse.lex"
/*
 * Copyright (c) 1996, 1998-2001 Todd C. Miller <Todd.Miller@courtesan.com>
 * All rights reserved.
 *
 * This code is derived from software contributed by Chris Jepeway
 * <jepeway@cs.utk.edu>
 *
 * This code is derived from software contributed by Chris Jepeway
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * 4. Products derived from this software may not be called "Sudo" nor
 *    may "Sudo" appear in their names without specific prior written
 *    permission from the author.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include <sys/types.h>
#include <sys/param.h>
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif /* STDC_HEADERS */
#ifdef HAVE_STRING_H
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif /* HAVE_STRING_H */
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif /* HAVE_UNISTD_H */
#if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
# include <malloc.h>
#endif /* HAVE_MALLOC_H && !STDC_HEADERS */
#include <ctype.h>
#include "sudo.h"
#include "parse.h"
#include <sudo.tab.h>

#ifndef lint
static const char rcsid[] = "$Sudo: lex.yy.c,v 1.36 2002/01/15 18:16:46 millert Exp $";
#endif /* lint */

#undef yywrap		/* guard against a yywrap macro */

extern YYSTYPE yylval;
extern int clearaliases;
int sudolineno = 1;
static int sawspace = 0;
static int arg_len = 0;
static int arg_size = 0;

static void fill		__P((char *, int));
static void fill_cmnd		__P((char *, int));
static void fill_args		__P((char *, int, int));
extern void reset_aliases	__P((void));
extern void yyerror		__P((char *));

/* realloc() to size + COMMANDARGINC to make room for command args */
#define COMMANDARGINC	64

#ifdef TRACELEXER
#define LEXTRACE(msg)	fputs(msg, stderr)
#else
#define LEXTRACE(msg)
#endif
/* XXX - convert GOTRUNAS to exclusive state (GOTDEFS cannot be) */
#define GOTRUNAS 1

#define GOTDEFS 2

#define GOTCMND 3

#define STARTDEFS 4

#define INDEFS 5

#line 985 "lex.yy.c"

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#ifdef __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		yy_current_buffer->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

#line 114 "parse.lex"

#line 1141 "lex.yy.c"

	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 299 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1808 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 115 "parse.lex"
BEGIN STARTDEFS;
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 117 "parse.lex"
{
			    BEGIN INDEFS;
			    LEXTRACE("DEFVAR ");
			    fill(yytext, yyleng);
			    return(DEFVAR);
			}
	YY_BREAK

case 3:
YY_RULE_SETUP
#line 125 "parse.lex"
{
			    BEGIN STARTDEFS;
			    LEXTRACE(", ");
			    return(',');
			}			/* return ',' */
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 131 "parse.lex"
{
			    LEXTRACE("= ");
			    return('=');
			}			/* return '=' */
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 136 "parse.lex"
{
			    LEXTRACE("+= ");
			    return('+');
			}			/* return '+' */
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 141 "parse.lex"
{
			    LEXTRACE("-= ");
			    return('-');
			}			/* return '-' */
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 146 "parse.lex"
{
			    LEXTRACE("WORD(1) ");
			    fill(yytext + 1, yyleng - 2);
			    return(WORD);
			}
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 152 "parse.lex"
{
			    LEXTRACE("WORD(2) ");
			    fill(yytext, yyleng);
			    return(WORD);
			}
	YY_BREAK


case 9:
YY_RULE_SETUP
#line 160 "parse.lex"
{
			    LEXTRACE("QUOTEDCHAR ");
			    fill_args(yytext + 1, 1, sawspace);
			    sawspace = FALSE;
			}
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 166 "parse.lex"
{
			    BEGIN INITIAL;
			    unput(*yytext);
			    return(COMMAND);
			}			/* end of command line args */
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 172 "parse.lex"
{
			    LEXTRACE("ARG ");
			    fill_args(yytext, yyleng, sawspace);
			    sawspace = FALSE;
			}			/* a command line arg */
	YY_BREAK

case 12:
YY_RULE_SETUP
#line 179 "parse.lex"
{
			    BEGIN GOTDEFS;
			    switch (yytext[8]) {
				case ':':
				    LEXTRACE("DEFAULTS_USER ");
				    return(DEFAULTS_USER);
				case '@':
				    LEXTRACE("DEFAULTS_HOST ");
				    return(DEFAULTS_HOST);
				default:
				    LEXTRACE("DEFAULTS ");
				    return(DEFAULTS);
			    }
			}
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 194 "parse.lex"
{
			    fill(yytext, yyleng);
			    switch (*yytext) {
				case 'H':
				    LEXTRACE("HOSTALIAS ");
				    return(HOSTALIAS);
				case 'C':
				    LEXTRACE("CMNDALIAS ");
				    return(CMNDALIAS);
				case 'U':
				    LEXTRACE("USERALIAS ");
				    return(USERALIAS);
				case 'R':
				    LEXTRACE("RUNASALIAS ");
				    BEGIN GOTRUNAS;
				    return(RUNASALIAS);
			    }
			}
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 213 "parse.lex"
{
				/* cmnd does not require passwd for this user */
			    	LEXTRACE("NOPASSWD ");
			    	return(NOPASSWD);
			}
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 219 "parse.lex"
{
				/* cmnd requires passwd for this user */
			    	LEXTRACE("PASSWD ");
			    	return(PASSWD);
			}
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 225 "parse.lex"
{
			    /* netgroup */
			    fill(yytext, yyleng);
			    LEXTRACE("NETGROUP ");
			    return(NETGROUP);
			}
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 232 "parse.lex"
{
			    /* UN*X group */
			    fill(yytext, yyleng);
			    LEXTRACE("GROUP ");
			    return(USERGROUP);
			}
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 239 "parse.lex"
{
			    fill(yytext, yyleng);
			    LEXTRACE("NTWKADDR ");
			    return(NTWKADDR);
			}
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 245 "parse.lex"
{
			    fill(yytext, yyleng);
			    LEXTRACE("NTWKADDR ");
			    return(NTWKADDR);
			}
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 251 "parse.lex"
{
				BEGIN GOTRUNAS;
				LEXTRACE("RUNAS ");
				return (RUNAS);
			}
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 257 "parse.lex"
{
			    if (strcmp(yytext, "ALL") == 0) {
				LEXTRACE("ALL ");
				return(ALL);
			    } else {
				fill(yytext, yyleng);
				LEXTRACE("ALIAS ");
				return(ALIAS);
			    }
			}
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 268 "parse.lex"
{
			    /* username/uid that user can run command as */
			    fill(yytext, yyleng);
			    LEXTRACE("WORD(3) ");
			    return(WORD);
			}
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 275 "parse.lex"
{
			    BEGIN INITIAL;
			}
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 279 "parse.lex"
{
			    /* directories can't have args... */
			    if (yytext[yyleng - 1] == '/') {
				LEXTRACE("COMMAND ");
				fill_cmnd(yytext, yyleng);
				return(COMMAND);
			    } else {
				BEGIN GOTCMND;
				LEXTRACE("COMMAND ");
				fill_cmnd(yytext, yyleng);
			    }
			}			/* a pathname */
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 292 "parse.lex"
{
			    /* a word */
			    fill(yytext, yyleng);
			    LEXTRACE("WORD(4) ");
			    return(WORD);
			}
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 299 "parse.lex"
{
			    LEXTRACE(", ");
			    return(',');
			}			/* return ',' */
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 304 "parse.lex"
{
			    LEXTRACE("= ");
			    return('=');
			}			/* return '=' */
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 309 "parse.lex"
{
			    LEXTRACE(": ");
			    return(':');
			}			/* return ':' */
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 314 "parse.lex"
{
			    if (yyleng % 2 == 1)
				return('!');	/* return '!' */
			}
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 319 "parse.lex"
{
			    BEGIN INITIAL;
			    ++sudolineno;
			    LEXTRACE("\n");
			    return(COMMENT);
			}			/* return newline */
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 326 "parse.lex"
{			/* throw away space/tabs */
			    sawspace = TRUE;	/* but remember for fill_args */
			}
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 330 "parse.lex"
{
			    sawspace = TRUE;	/* remember for fill_args */
			    ++sudolineno;
			    LEXTRACE("\n\t");
			}			/* throw away EOL after \ */
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 336 "parse.lex"
{
			    BEGIN INITIAL;
			    ++sudolineno;
			    LEXTRACE("\n");
			    return(COMMENT);
			}			/* return comments */
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 343 "parse.lex"
{
			    LEXTRACE("ERROR ");
			    return(ERROR);
			}	/* parse error */
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 348 "parse.lex"
ECHO;
	YY_BREAK
#line 1565 "lex.yy.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(GOTRUNAS):
case YY_STATE_EOF(GOTDEFS):
case YY_STATE_EOF(GOTCMND):
case YY_STATE_EOF(STARTDEFS):
case YY_STATE_EOF(INDEFS):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_current_buffer->yy_n_chars = yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );

		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 299 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 299 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 298);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_current_buffer->yy_n_chars =
			yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yy_c_buf_p - yytext_ptr;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						return EOF;

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	yy_current_buffer->yy_at_bol = (c == '\n');

	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int isatty YY_PROTO(( int ));
#endif
#endif

#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	int oerrno = errno;

	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	errno = oerrno;
	}


#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b )
#else
void yy_flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size )
char *base;
yy_size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *yy_str )
#else
YY_BUFFER_STATE yy_scan_string( yy_str )
yyconst char *yy_str;
#endif
	{
	int len;
	for ( len = 0; yy_str[len]; ++len )
		;

	return yy_scan_bytes( yy_str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len )
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len )
yyconst char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void yy_fatal_error( yyconst char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s )
#else
static int yy_flex_strlen( s )
yyconst char *s;
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
yy_size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	yylex();
	return 0;
	}
#endif
#line 348 "parse.lex"

static void
fill(s, len)
    char *s;
    int len;
{
    int i, j;

    yylval.string = (char *) malloc(len + 1);
    if (yylval.string == NULL)
	yyerror("unable to allocate memory");

    /* Copy the string and collapse any escaped characters. */
    for (i = 0, j = 0; i < len; i++, j++) {
	if (s[i] == '\\' && i != len - 1)
	    yylval.string[j] = s[++i];
	else
	    yylval.string[j] = s[i];
    }
    yylval.string[j] = '\0';
}

static void
fill_cmnd(s, len)
    char *s;
    int len;
{
    arg_len = arg_size = 0;

    yylval.command.cmnd = (char *) malloc(len + 1);
    if (yylval.command.cmnd == NULL)
	yyerror("unable to allocate memory");

    /* copy the string and NULL-terminate it (escapes handled by fnmatch) */
    (void) strncpy(yylval.command.cmnd, s, len);
    yylval.command.cmnd[len] = '\0';

    yylval.command.args = NULL;
}

static void
fill_args(s, len, addspace)
    char *s;
    int len;
    int addspace;
{
    int new_len;
    char *p;

    /*
     * If first arg, malloc() some room, else if we don't
     * have enough space realloc() some more.
     */
    if (yylval.command.args == NULL) {
	addspace = 0;
	new_len = len;

	while (new_len >= (arg_size += COMMANDARGINC))
	    ;

	yylval.command.args = (char *) malloc(arg_size);
	if (yylval.command.args == NULL)
	    yyerror("unable to allocate memory");
    } else {
	new_len = arg_len + len + addspace;

	if (new_len >= arg_size) {
	    /* Allocate more space than we need for subsequent args */
	    while (new_len >= (arg_size += COMMANDARGINC))
		;

	    if ((p = (char *) realloc(yylval.command.args, arg_size)) == NULL) {
		free(yylval.command.args);
		yyerror("unable to allocate memory");
	    } else
		yylval.command.args = p;
	}
    }

    /* Efficiently append the arg (with a leading space if needed). */
    p = yylval.command.args + arg_len;
    if (addspace)
	*p++ = ' ';
    (void) strcpy(p, s);
    arg_len = new_len;
}

int
yywrap()
{

    /* Free space used by the aliases unless called by testsudoers. */
    if (clearaliases)
	reset_aliases();

    return(TRUE);
}
