#define I_SYSLOG_IFAC           0
#define I_SYSLOG_IGOODPRI       1
#define I_SYSLOG_IBADPRI        2
#define I_SYSLOG                3
#define I_SYSLOG_GOODPRI        4
#define I_SYSLOG_BADPRI         5
#define I_LONG_OTP_PROMPT       6
#define I_IGNORE_DOT            7
#define I_MAIL_ALWAYS           8
#define I_MAIL_BADPASS          9
#define I_MAIL_NO_USER          10
#define I_MAIL_NO_HOST          11
#define I_MAIL_NO_PERMS         12
#define I_TTY_TICKETS           13
#define I_LECTURE               14
#define I_AUTHENTICATE          15
#define I_ROOT_SUDO             16
#define I_LOG_HOST              17
#define I_LOG_YEAR              18
#define I_SHELL_NOARGS          19
#define I_SET_HOME              20
#define I_ALWAYS_SET_HOME       21
#define I_PATH_INFO             22
#define I_FQDN                  23
#define I_INSULTS               24
#define I_REQUIRETTY            25
#define I_ENV_EDITOR            26
#define I_ROOTPW                27
#define I_RUNASPW               28
#define I_TARGETPW              29
#define I_USE_LOGINCLASS        30
#define I_SET_LOGNAME           31
#define I_STAY_SETUID           32
#define I_ENV_RESET             33
#define I_PRESERVE_GROUPS       34
#define I_LOGLINELEN            35
#define I_TIMESTAMP_TIMEOUT     36
#define I_PASSWD_TIMEOUT        37
#define I_PASSWD_TRIES          38
#define I_UMASK                 39
#define I_LOGFILE               40
#define I_MAILERPATH            41
#define I_MAILERFLAGS           42
#define I_MAILTO                43
#define I_MAILSUB               44
#define I_BADPASS_MESSAGE       45
#define I_TIMESTAMPDIR          46
#define I_EXEMPT_GROUP          47
#define I_PASSPROMPT            48
#define I_RUNAS_DEFAULT         49
#define I_EDITOR                50
#define I_ENV_CHECK             51
#define I_ENV_DELETE            52
#define I_ENV_KEEP              53
#define I_LISTPW_I              54
#define I_VERIFYPW_I            55
#define I_LISTPW                56
#define I_VERIFYPW              57
